import { Injectable } from '@nestjs/common';
import { InjectEntityManager, InjectRepository } from '@nestjs/typeorm';
import { MaterialsInventoryEntity } from './materials_inventory.entity';
import { Storage } from '../tools/storage/storage.entity';
import { EntityManager, Repository } from 'typeorm';
import {
  MaterialsInventoryDto,
  MaterialsInventoryQueryDto,
  MaterialsInventoryUpdateDto
} from './materials_inventory.dto';
import { Pagination } from '~/helper/paginate/pagination';
import { BusinessException } from '~/common/exceptions/biz.exception';
import { ErrorEnum } from '~/constants/error-code.constant';
import { paginate } from '~/helper/paginate';

@Injectable()
export class MaterialsInventoryService {
  constructor(
    @InjectEntityManager() private entityManager: EntityManager,
    @InjectRepository(MaterialsInventoryEntity)
    private materialsInventoryRepository: Repository<MaterialsInventoryEntity>,
    @InjectRepository(Storage)
    private storageRepository: Repository<Storage>
  ) {}
  /**
   * 列举所有角色：除去超级管理员
   */
  async findAll({
    page,
    pageSize
    // materialsInventoryNumber,
    // title,
    // type,
    // status
  }: MaterialsInventoryQueryDto): Promise<Pagination<MaterialsInventoryEntity>> {
    const queryBuilder = this.materialsInventoryRepository
      .createQueryBuilder('materialsInventory')
      .leftJoin('materialsInventory.files', 'files')
      .addSelect(['files.id', 'files.path'])
      //   .where({
      //     ...(materialsInventoryNumber ? { materialsInventoryNumber: Like(`%${materialsInventoryNumber}%`) } : null),
      //     ...(title ? { title: Like(`%${title}%`) } : null),
      //     ...(isNumber(type) ? { type } : null),
      //     ...(isNumber(status) ? { status } : null)
      //   })
      .andWhere('materialsInventory.isDelete = 0');

    return paginate<MaterialsInventoryEntity>(queryBuilder, {
      page,
      pageSize
    });
  }

  /**
   * 新增
   */
  async create(dto: MaterialsInventoryDto): Promise<void> {
    await this.materialsInventoryRepository.insert(dto);
  }

  /**
   * 更新
   */
  async update(
    id: number,
    { fileIds, ...data }: Partial<MaterialsInventoryUpdateDto>
  ): Promise<void> {
    await this.entityManager.transaction(async manager => {
      await manager.update(MaterialsInventoryEntity, id, {
        ...data
      });
      const materialsInventory = await this.materialsInventoryRepository
        .createQueryBuilder('materialsInventory')
        .leftJoinAndSelect('materialsInventory.files', 'files')
        .where('materialsInventory.id = :id', { id })
        .getOne();
      if (fileIds?.length) {
        const count = await this.storageRepository
          .createQueryBuilder('storage')
          .where('storage.id in(:fileIds)', { fileIds })
          .getCount();
        if (count !== fileIds?.length) {
          throw new BusinessException(ErrorEnum.STORAGE_NOT_FOUND);
        }
        // 附件要批量更新
        await manager
          .createQueryBuilder()
          .relation(MaterialsInventoryEntity, 'files')
          .of(id)
          .addAndRemove(fileIds, materialsInventory.files);
      }
    });
  }

  /**
   * 删除
   */
  async delete(id: number): Promise<void> {
    // 合同比较重要，做逻辑删除
    await this.materialsInventoryRepository.update(id, { isDelete: 1 });
  }

  /**
   * 获取单个合同信息
   */
  async info(id: number) {
    const info = await this.materialsInventoryRepository
      .createQueryBuilder('materialsInventory')
      .where({
        id
      })
      .andWhere('materialsInventory.isDelete = 0')
      .getOne();
    return info;
  }

  /**
   * 解除附件关联
   * @param id 合同ID
   * @param fileIds 附件ID
   */
  async unlinkAttachments(id: number, fileIds: number[]) {
    await this.entityManager.transaction(async manager => {
      const materialsInventory = await this.materialsInventoryRepository
        .createQueryBuilder('materialsInventory')
        .leftJoinAndSelect('materialsInventory.files', 'files')
        .where('materialsInventory.id = :id', { id })
        .getOne();
      const linkedFiles = materialsInventory.files
        .map(item => item.id)
        .filter(item => !fileIds.includes(item));
      // 附件要批量更新
      await manager
        .createQueryBuilder()
        .relation(MaterialsInventoryEntity, 'files')
        .of(id)
        .addAndRemove(linkedFiles, materialsInventory.files);
    });
  }
}
